﻿namespace WpfApplication2.Controls
{
  using System.Collections.Generic;
  using System.Diagnostics;
  using System.Windows.Controls;
  using System.Linq;
  using System.Collections.ObjectModel;

  public partial class HandInformationControl : UserControl, ISampleRenderer
  {
    public HandInformationControl()
    {
      InitializeComponent();
    }
    public void Initialise(PXCMSenseManager senseManager)
    {
      this.senseManager = senseManager;

      this.senseManager.EnableHand().ThrowOnFail();

      using (var handModule = this.senseManager.QueryHand())
      {
        using (var handConfiguration = handModule.CreateActiveConfiguration())
        {
          this.alertManager = new HandAlertManager(handConfiguration);

          handConfiguration.EnableAllGestures().ThrowOnFail();

          handConfiguration.ApplyChanges().ThrowOnFail();
        }
        this.handData = handModule.CreateOutput();
      }
    }
    public void ProcessSampleWorkerThread(PXCMCapture.Sample sample)
    {
      this.handStatusMap = null;

      if (this.handData.Update().Succeeded())
      {
        var handsInfoFromAlerts = this.alertManager.GetHandsInfo();

        if ((handsInfoFromAlerts != null) && (handsInfoFromAlerts.Count() > 0))
        {
          this.handStatusMap = new Dictionary<int, HandAlertManager.HandStatus>(
            handsInfoFromAlerts);

          this.HandleGestures();
        }       
      }
    }
    void HandleGestures()
    {
      var handsAtGoodStatus =
        this.handStatusMap
        .Where(kvp => kvp.Value == HandAlertManager.HandStatus.Ok);

      if (handsAtGoodStatus.Count() > 0)
      {
        if (this.gestureTrackingHandId != handsAtGoodStatus.First().Key)
        {
          this.gestureTrackingHandId = handsAtGoodStatus.First().Key;
          this.gesturesTracked = null;
        }
        var gestureCount = this.handData.QueryFiredGesturesNumber();
        PXCMHandData.GestureData gestureData;

        if (gestureCount > 0)
        {
          for (int i = 0; i < gestureCount; i++)
          {
            if (this.handData.QueryFiredGestureData(i, out gestureData).Succeeded())
            {
              if ((gestureData.handId == this.gestureTrackingHandId) &&
                (gestureData.state == PXCMHandData.GestureStateType.GESTURE_STATE_END))
              {
                if (this.gesturesTracked == null)
                {
                  this.gesturesTracked = new List<string>();
                }
                this.gesturesTracked.Insert(0, gestureData.name);
              }
            }
          }
        }
      }
      else
      {
        this.gestureTrackingHandId = -1;
        this.gesturesTracked = null;
      }
    }
    public void RenderUI(PXCMCapture.Sample sample)
    {
      this.listHandStatus.ItemsSource = this.handStatusMap;
      
      // very hacky, I should really do all this properly and use binding
      // and observable collections and so on but for experimentation
      // purposes I get away with it...
      this.listGestures.ItemsSource = null;
      this.listGestures.ItemsSource = this.gesturesTracked;
    }
    int gestureTrackingHandId;
    List<string> gesturesTracked;
    Dictionary<int, HandAlertManager.HandStatus> handStatusMap;
    HandAlertManager alertManager;
    PXCMSenseManager senseManager;
    PXCMHandData handData;
  }
}