﻿namespace WpfApplication2
{
  using System;
  using System.Collections.Generic;
  using System.Windows;
  using System.Linq;
  using System.Diagnostics;
  using System.Threading;
  using System.Windows.Threading;
  using System.Threading.Tasks;

  public partial class MainWindow : Window
  {
    public MainWindow()
    {
      InitializeComponent();
      this.cancelTokenSource = new CancellationTokenSource();
      this.Loaded += OnLoaded;
      this.Closing += OnClosing;      
    }
    void OnClosing(object sender, System.ComponentModel.CancelEventArgs e)
    {
      this.cancelTokenSource.Cancel();
    }
    void OnLoaded(object sender, RoutedEventArgs e)
    {
      this.senseManager = PXCMSenseManager.CreateInstance();

      this.senseManager.captureManager.SetRealtime(false);

      this.senseManager.EnableStream(PXCMCapture.StreamType.STREAM_TYPE_COLOR, 1280, 720);

      this.InitialiseRenderers();

      this.senseManager.Init().ThrowOnFail();

      this.RunTask();
    }
    async void RunTask()
    {
      try
      {
        await Task.Run(async () =>
        {
          while (true)
          {
            this.cancelTokenSource.Token.ThrowIfCancellationRequested();

            if (this.senseManager.AcquireFrame(true).Succeeded())
            {
              var sample = this.senseManager.QuerySample();

              this.ForAllRenderers(r => r.ProcessSampleWorkerThread(sample));

              await this.Dispatcher.InvokeAsync(
                () =>
                {
                  this.ForAllRenderers(r => r.RenderUI(sample));
                }
              );
              this.senseManager.ReleaseFrame();
            }
          }
        });
      }
      catch (OperationCanceledException)
      {
      }
    }
    void InitialiseRenderers()
    {
      this.renderers = this.BuildRenderers();

      foreach (var renderer in this.renderers)
      {
        renderer.Initialise(this.senseManager);
      }
    }
    void ForAllRenderers(Action<ISampleRenderer> action)
    {
      foreach (var renderer in this.renderers)
      {
        action(renderer);
      }
    }
    List<ISampleRenderer> BuildRenderers()
    {
      List<ISampleRenderer> list = new List<ISampleRenderer>();

      foreach (var control in this.parentGrid.Children)
      {
        ISampleRenderer renderer = control as ISampleRenderer;
        if (renderer != null)
        {
          list.Add(renderer);
        }
      }
      return (list);
    }
    CancellationTokenSource cancelTokenSource;
    IEnumerable<ISampleRenderer> renderers;
    PXCMSenseManager senseManager;
  }
}